@@ -17,8 +17,8 @@
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-	$Id: fdcopy.c 7751 2005-06-01 09:01:41Z wsl $
-	$URL: https://infix.uvt.nl/its-id/trunk/sources/fair/src/fdcopy.c $
+	$Id: fdcopy.c 23144 2007-12-21 15:36:48Z wsl $
+	$URL: https://infix.uvt.nl/its-id/trunk/sources/fair-0.5.1/src/fdcopy.c $
 
 ******************************************************************************/
 
