@@ -17,8 +17,8 @@
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-	$Id: transponder.c 7718 2005-05-27 08:24:55Z wsl $
-	$URL: https://infix.uvt.nl/its-id/trunk/sources/fair/src/transponder.c $
+	$Id: transponder.c 23144 2007-12-21 15:36:48Z wsl $
+	$URL: https://infix.uvt.nl/its-id/trunk/sources/fair-0.5.1/src/transponder.c $
 
 ******************************************************************************/
 
@@ -221,7 +221,7 @@ static void readload(void) {
 	buf[r] = '\0';
 	if(sscanf(buf, "%lf %lf %lf ", &l1, &l5, &l15) != 3)
 		return;
-	msg->capacity = htons(max_capacity / pow(2.0, l1));
+	msg->capacity = htons(max_capacity / exp(l1));
 }
 
 int main(int argc, char **argv) {
@@ -293,7 +293,7 @@ int main(int argc, char **argv) {
 			spamall(FALSE, conf_PingInterval);
 		if(init_hupped) {
 			init_hupped = FALSE;
-			if(conf_reload(NULL))
+			if(conf_reload(config))
 				syslog(LOG_INFO, "Config reloaded.");
 			else
 				syslog(LOG_ERR, "Errors in config file, not reloaded");
