@@ -17,8 +17,8 @@
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-	$Id: worker.c 7753 2005-06-01 10:45:34Z wsl $
-	$URL: https://infix.uvt.nl/its-id/trunk/sources/fair/src/worker.c $
+	$Id: worker.c 23144 2007-12-21 15:36:48Z wsl $
+	$URL: https://infix.uvt.nl/its-id/trunk/sources/fair-0.5.1/src/worker.c $
 
 ******************************************************************************/
 
@@ -36,6 +36,7 @@
 #include "error.h"
 #include "conf.h"
 #include "address.h"
+#include "avlr.h"
 
 #include "worker.h"
 
@@ -70,12 +71,14 @@ static int worker_cmp_name(const worker_t *aw, const worker_t *bw) {
 }
 
 static int worker_cmp_best(const worker_t *aw, const worker_t *bw) {
+	avl_node_t *n;
 	bool ab, bb;
 	u32 ar, br;
-	int ac, bc;
 	address_t *aa, *ba;
+#if 0
+	int ac, bc;
 	sa_family_t af, bf;
-	avl_node_t *n;
+#endif
 
 	assert(aw && bw);
 
@@ -127,6 +130,7 @@ static int worker_cmp_best(const worker_t *aw, const worker_t *bw) {
 			return 1;
 	}
 
+#if 0
 	/* if both have equal load, start with the strongest */
 	ac = aw->capacity;
 	bc = bw->capacity;
@@ -149,23 +153,11 @@ static int worker_cmp_best(const worker_t *aw, const worker_t *bw) {
 		return -1;
 	if(aw < bw)
 		return 1;
+#endif
 
 	return 0;
 }
 
-static avl_node_t *avl_insert_node_somewhere(avl_tree_t *avltree, avl_node_t *newnode) {
-	avl_node_t *node;
-
-	assert(avltree && newnode);
-
-	if(!avltree->top)
-		return avl_insert_top(avltree, newnode);
-
-	return avl_search_closest(avltree, newnode->item, &node) == 1
-		? avl_insert_after(avltree, node, newnode)
-		: avl_insert_before(avltree, node, newnode);
-}
-
 static avl_tree_t workers_name = {NULL, NULL, NULL, (avl_compare_t)worker_cmp_name, NULL};
 static avl_tree_t workers_best = {NULL, NULL, NULL, (avl_compare_t)worker_cmp_best, NULL};
 
@@ -176,33 +168,34 @@ unsigned int worker_count(void) {
 static void worker_changed(worker_t *wrk) {
 	unless(wrk) return;
 	avl_unlink_node(&workers_best, &wrk->best);
-	avl_insert_node_somewhere(&workers_best, &wrk->best);
+	avl_insert_right(&workers_best, &wrk->best);
 }
 
 static void address_changed(address_t *addr, address_event_t evt) {
 	worker_t *wrk;
+
 	unless(addr) return;
 	wrk = addr->data;
 	unless(wrk) return;
 
 	avl_unlink_node(&wrk->prio, &addr->prio);
-	avl_insert_node_somewhere(&wrk->prio, &addr->prio);
+	avl_insert_right(&wrk->prio, &addr->prio);
 
 	if(evt == ADDRESS_EVENT_LOAD) {
 		wrk->nconn++;
 		wrk->hysteresis = 0;
+		worker_changed(wrk);
+		/* can't change two workers at the same time */
+		if(wrk != worker_last) {
+			if(worker_last) {
+				worker_last->hysteresis = conf_Hysteresis;
+				worker_changed(worker_last);
+			}
+			worker_last = wrk;
+		}
 	} else if(evt == ADDRESS_EVENT_UNLOAD) {
 		wrk->nconn--;
-	}
-	worker_changed(wrk);
-
-	/* can't change two workers at the same time */
-	if(evt == ADDRESS_EVENT_LOAD && wrk != worker_last) {
-		if(worker_last) {
-			worker_last->hysteresis = conf_Hysteresis;
-			worker_changed(worker_last);
-		}
-		worker_last = wrk;
+		worker_changed(wrk);
 	}
 }
 
@@ -220,7 +213,7 @@ worker_t *worker_new(const char *name) {
 	avl_init_tree(&wrk->addr, (avl_compare_t)address_cmp_addr, NULL);
 	avl_init_tree(&wrk->prio, (avl_compare_t)address_cmp_prio, NULL);
 	avl_insert_node(&workers_name, &wrk->node);
-	avl_insert_node_somewhere(&workers_best, &wrk->best);
+	avl_insert_right(&workers_best, &wrk->best);
 	syslog(LOG_INFO, "Added new worker node %s", name);
 	return wrk;
 }
@@ -229,9 +222,10 @@ worker_t *worker_byname(const char *name) {
 	avl_node_t *n;
 	worker_t *wrk;
 	size_t len;
-	assert(name);
+	unless(name) return NULL;
 	len = strlen(name);
 	wrk = alloca(sizeof *wrk + len);
+	unless(wrk) return NULL;
 	*wrk = worker_0;
 	strcpy(wrk->name, name);
 	n = avl_search(&workers_name, wrk);
@@ -257,9 +251,10 @@ void worker_update(worker_t *wrk, int capacity, const struct sockaddr *sa, sockl
 		addr = address_new(sa, len, address_changed, wrk);
 		assert(addr);
 		avl_insert_node(&wrk->addr, &addr->node);
-		avl_insert_node_somewhere(&wrk->prio, &addr->prio);
+		avl_insert_right(&wrk->prio, &addr->prio);
 		address_string_sa(&str, sa, len);
-		syslog(LOG_INFO, "Added new worker address %s to %s", str.host, wrk->name);
+		syslog(LOG_INFO, "Added new worker address %s %s to %s",
+			str.host, str.serv, wrk->name);
 		worker_changed(wrk);
 	}
 }
@@ -268,22 +263,27 @@ static void worker_print(const worker_t *wrk, const address_t *best) {
 	avl_node_t *n;
 	address_t *addr;
 
+	unless(wrk) return;
+
 	eprintf("\r%s: capacity:%d conn:%d score:%llu\033[K\n", wrk->name,
-		(int)wrk->capacity, (int)wrk->nconn, (unsigned long long)worker_prio(wrk));
+		(int)wrk->capacity, (int)wrk->nconn,
+		(unsigned long long)worker_prio(wrk));
 
 	for(n = wrk->addr.head; n; n = n->next) {
 		addr = n->item;
 		assert(addr);
-		eprintf("\tconn:%d errors:%d dead:%d disabled:%d%s\n",
-			(int)addr->nconn, (int)addr->errors, 
+		eprintf("%s\tconn:%d errors:%d dead:%d disabled:%d %s %s\n",
+			addr == best ? " *" : "",
+			(int)addr->nconn, (int)addr->errors,
 			(int)addr->dead, (int)addr->disabled,
-			addr == best ? " *" : "");
+			addr->str.host, addr->str.serv);
 	}
 }
 
 address_t *worker_bestaddress(void) {
 	avl_node_t *n;
 	worker_t *wrk;
+	address_t *addr;
 	n = workers_best.head;
 	if(!n)
 		return NULL;
@@ -292,8 +292,9 @@ address_t *worker_bestaddress(void) {
 	n = wrk->prio.head;
 	if(!n)
 		return NULL;
+	addr = n->item;
 	if(conf_Debug)
 		for(n = workers_best.head; n; n = n->next)
-			worker_print(n->item, n->item);
-	return n->item;
+			worker_print(n->item, addr);
+	return addr;
 }
